var MapGame = cc.Class.extend({

    ctor: function () {
        this.keys = new Array();
        this.data = new Object();
    },


    /**
     * 放入一个键值对
     * @param {String} key
     * @param {Object} value
     */
    put: function (key, value) {
        this.remove(key);
        this.keys.push(key);
        this.data[key] = value;

        this.sort();

        return true;
    },

    //排序
    sort: function () {
        //this.keys.sort();
        this.keys.sort(function(a,b)
        {
            return b < a
        });


    },


    begin: function () {
        var len = this.keys.length;
        for (var i = 0; i < len; i++) {
            var k = this.keys[i];
            return this.data[k];
        }

        return null;
    },
    //
    pop: function () {
        var len = this.keys.length;
        for (var i = 0; i < len; i++) {
            var k = this.keys[i];
            this.remove(k);
            return true;
           // return this.data[k];
        }
    },

    end: function () {
        return null;
    },


    getnext: function (key) {
        for (var i = 0; i < this.keys.length; i++) {
            if (key === this.keys[i]) {
                if (i + 1 < this.keys.length) {
                    var k = this.keys[i + 1];
                    var nodeinfo = this.data[k];
                    return nodeinfo;
                }
            }
        }
        return null;
    },


    /**
     * 获取某键对应的值
     * @param {String} key
     * @return {Object} value
     */
    get: function (key) {
        return this.data[key];
    },

    getData: function (index) {
        if (index == null) {
            return null;
        }
        var k = this.keys[index];
        var nodeinfo = this.data[k];
        return nodeinfo;
    },
    getDataByKey: function (key) {
        for (var i = 0; i < this.keys.length; i++) {
            if (key === this.keys[i]) {
                return this.data[key];
            }
        }
        return null;
    },

    hasData: function (key) {
        for (var i = 0; i < this.keys.length; i++) {
            if (key === this.keys[i]) {

                return true;

            }
        }
        return false;
    },

    //删除所有数据
    clear: function () {
        this.keys = null;
        this.data = null;
        this.keys = new Array();
        this.data = new Object();
    },
    /**
     * 删除一个键值对
     * @param {String} key
     */
    remove: function (key) {
        for (var i = 0; i < this.keys.length; i++) {
            if (key === this.keys[i]) {
                var del_keys = this.keys.splice(i, 1);
                for (k in del_keys) {
                    this.data[k] = null;
                }
                return true;
            }
        }
        return false;
    },

    /**
     * 遍历Map,执行处理函数
     *
     * @param {Function} 回调函数 function(key,value,index){..}
     */



    each: function (fn, point, data) {
        if (typeof fn != 'function') {
            return;
        }
        this.m_ponit = point;

        var len = this.keys.length;
        for (var i = 0; i < len; i++) {
            var k = this.keys[i];
            fn.call(this.m_ponit, k, this.data[k], data);
        }
    },

    /**
     * 获取键值数组
     * @return entity[{key,value},{key,value}]
     */
    entrySet: function () {
        var len = this.keys.length;
        var entrys = new Array(len);
        for (var i = 0; i < len; i++) {
            entrys[i] = {
                key: this.keys[i],
                value: this.data[this.keys[i]]
            };
        }
        return entrys;
    },

    /**
     * 判断Map是否为空
     */
    isEmpty: function () {
        return this.keys.length == 0;
    },
    empty: function () {
        return this.keys.length == 0;
    },
    /**
     * 获取键值对数量
     */
    size: function () {
        return this.keys.length;
    },
    getSize: function () {
        return this.keys.length;
    },


    containsKey: function (key) {
        return this.keys.filter(function (v) {
                if (v === key) {
                    return key;
                }
            }).length > 0;
    },
    /**
     * 重写toString
     */
    toString: function () {
        var s = "{";
        for (var i = 0; i < this.keys.length; i++) {
            var k = this.keys[i];
            s += k + "=" + this.data[k];
            if (this.keys.length > i + 1) {
                s += ','
            }
        }
        s += "}";
        return s;
    },
    /**
     * 解析字符串到Map
     * {a=A,b=B,c=B,}
     */
    parserStringAndAddMap: function (str) {
        var count = 0;
        if (str && str.length > 0) {
            str = str.trim();
            var startIndex = str.indexOf("{"), endIndex = str.lastIndexOf("}");
            if (startIndex !== -1 && endIndex !== -1) {
                str = str.substring(startIndex + 1, endIndex);
                var arrs = str.split(",");
                for (var i = 0; i < arrs.length; i++) {
                    var kv = arrs[i].trim();
                    if (kv.length > 0 && kv.indexOf("=") !== -1) {
                        var kv_arr = kv.split("=");
                        if (kv_arr.length == 2) {
                            if (this.put(kv_arr[0].trim(), kv_arr[1].trim())) {
                                count++;
                            } else {
                                console.error('error: kv:' + kv);
                            }

                        }
                    }
                }
            } else {
                console.log("data error:" + str);
            }
        } else {
            console.log('data is not empty');
        }
        return count;
    }

});

